<?php
	
	namespace Infocob\CRM\Products\Admin\Classes;
	
	use Infocob\CRM\Products\Admin\Classes\Infocob\Tools\DateTimeFr;
	use Monolog\Formatter\LineFormatter;
	use Monolog\Handler\StreamHandler;
	use Monolog\Logger as Monolog;
	
	if(!defined('ABSPATH') && !is_admin()) {
		die();
	}
	
	class Logger {
		private static string $info_path = "logs/imports/infos";
		private static string $error_path = "logs/imports/errors";
		
		/**
		 * @param string $message
		 * @param array  $vars
		 */
		public static function infoImport(string $message, array $vars = [], $lineBreak = false) {
			$datetime = new DateTimeFr();
			$log_filename = $datetime->format("d-m-Y");
			
			$dateFormat = "d/m/Y H:i:s";
			if($lineBreak) {
				$output = "[%datetime%] %level_name%: %message% %context%\n\r"; // Default output = [%datetime%] %channel%.%level_name%: %message% %context% %extra%\n
			} else {
				$output = "[%datetime%] %level_name%: %message% %context%\n"; // Default output = [%datetime%] %channel%.%level_name%: %message% %context% %extra%\n
			}
			$formatter = new LineFormatter($output, $dateFormat);
			
			// create a log channel
			$log = new Monolog('import');
			$stream_info = new StreamHandler(trim(ROOT_INFOCOB_CRM_PRODUCTS_DIR_PATH, "/") . "/" . Logger::$info_path . "/" . $log_filename . ".log", Monolog::INFO);
			$stream_info->setFormatter($formatter);
			$log->pushHandler($stream_info);
			
			$log->info($message, $vars);
		}
		
		/**
		 * @param string $message
		 * @param array  $vars
		 */
		public static function errorImport(string $message, array $vars = []) {
			$datetime = new DateTimeFr();
			$log_filename = $datetime->format("d-m-Y");
			
			$dateFormat = "d/m/Y H:i:s";
			$output = "[%datetime%] %level_name%: %message% %context%\n"; // Default output = [%datetime%] %channel%.%level_name%: %message% %context% %extra%\n
			$formatter = new LineFormatter($output, $dateFormat);
			
			// create a log channel
			$log = new Monolog('import');
			$stream_error = new StreamHandler(trim(ROOT_INFOCOB_CRM_PRODUCTS_DIR_PATH, "/") . "/" . Logger::$error_path . "/" . $log_filename . ".log", Monolog::ERROR);
			$stream_error->setFormatter($formatter);
			$log->pushHandler($stream_error);
			
			$log->error($message, $vars);
		}
		
		/**
		 * Delete old log files
		 */
		public static function cleanLogs($force = false) {
			if($force) {
				$expiration_time = time(); // Clean now
			} else {
				$expiration_time = time() - (60 * 60 * 24 * 7 * 26); // 13 weeks (~ 3 months)
			}
			
			foreach (glob(trim(ROOT_INFOCOB_CRM_PRODUCTS_DIR_PATH, "/") . "/" . Logger::$info_path . "/*.log") as $filename) {
				if(filemtime($filename) < $expiration_time) {
					unlink($filename);
				}
			}
			
			foreach (glob(trim(ROOT_INFOCOB_CRM_PRODUCTS_DIR_PATH, "/") . "/" . Logger::$error_path . "/*.log") as $filename) {
				if(filemtime($filename) < $expiration_time) {
					unlink($filename);
				}
			}
		}
		
		public static function getLogs() {
			$base_path = trim(ROOT_INFOCOB_CRM_PRODUCTS_DIR_PATH, "/") . "/logs/imports/";
			
			$logs_files = [];
			foreach(["infos", "errors"] as $level) {
				if(file_exists($base_path . $level)) {
					foreach (glob($base_path . "infos/*.log") as $path) {
						$logs_files[$level][] = [
							"path" => $path,
							"filename" => basename($path),
							"filename_without_ext" => pathinfo(basename($path), PATHINFO_FILENAME)
						];
					}
				}
				
				if(!empty($logs_files[$level])) {
					usort($logs_files[ $level ], function($file_a, $file_b) {
						$updatetime_a = filemtime($file_a["path"]);
						$updatetime_b = filemtime($file_b["path"]);
						if($updatetime_a == $updatetime_b) {
							return 0;
						}
						
						return ($updatetime_a > $updatetime_b) ? - 1 : 1;
					});
				}
			}
			
			return $logs_files;
		}
		
	}
